package eu.etaxonomy.taxeditor.session;

import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;
import org.junit.Assert;

import eu.etaxonomy.cdm.api.application.CdmChangeEvent;
import eu.etaxonomy.cdm.api.application.ICdmChangeListener;
import eu.etaxonomy.cdm.model.common.CdmBase;

public class MockSessionOwner<T extends CdmBase> implements ICdmEntitySessionEnabled, ICdmChangeListener {

    private static final Logger logger = Logger.getLogger(MockSessionOwner.class);

    private final List<T> rootEntities;
    private Set<T> expectedUpdatedObjects;
    private Set<? extends CdmBase> eventUpdatedObjects;

    private ICdmEntitySession session;

    public MockSessionOwner() {
        this.rootEntities = null;
        this.expectedUpdatedObjects = new HashSet<T>();
    }

//    public MockSessionOwner(Set<T> updatedObjects) {
//        this.rootEntities = null;
//        this.updatedObjects = updatedObjects;
//    }
//
//    public MockSessionOwner(List<T> rootEntities) {
//        this.rootEntities = rootEntities;
//    }

    public void setExpectedUpdatedObjects(Set<T> updatedObjects) {
        this.expectedUpdatedObjects = updatedObjects;
    }

    public void addExpectedUpdatedObject(T updatedObject) {
        this.expectedUpdatedObjects.add(updatedObject);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled#getCdmEntitySession()
     */
    @Override
    public ICdmEntitySession getCdmEntitySession() {
        return session;
    }

    public void setCdmEntitySession(ICdmEntitySession session) {
        this.session = session;;
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled#getRootEntities()
     */
    @Override
    public  List<T> getRootEntities() {
        return rootEntities;
    }

    public void dispose() {
        if(session != null) {
            session.dispose();
            logger.info("disposed of session from session owner : " + this);
        }
    }

    public Set<? extends CdmBase> getEventUpdatedObjects() {
        return eventUpdatedObjects;
    }
    /* (non-Javadoc)
     * @see eu.etaxonomy.cdm.api.application.ICdmChangeListener#onChange(eu.etaxonomy.cdm.api.application.CdmChangeEvent)
     */
    @Override
    public void onChange(CdmChangeEvent event) {
        eventUpdatedObjects = event.getChangedObjects();
        if(expectedUpdatedObjects != null && !expectedUpdatedObjects.isEmpty()) {
            for(CdmBase cb : expectedUpdatedObjects) {
                Assert.assertTrue(event.getChangedObjects().contains(cb));
            }
        }

    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled#getPropertyPathsMap()
     */
    @Override
    public Map<Object, List<String>> getPropertyPathsMap() {
        // TODO Auto-generated method stub
        return null;
    }

}
